LinkedList subclass: #SortedLinkedList
	instanceVariableNames: 'sortBlock '
	classVariableNames: 'DefaultSortBlock '
	poolDictionaries: ''
	category: 'Chapter 11'!


!SortedLinkedList methodsFor: 'initialization'!

initialize
	"Set the initial value of the receiver's sorting algorithm to a default."

	sortBlock := DefaultSortBlock!

sortBlock: aBlock
	"Set the initial value of the receiver's sorting algorithm."

	sortBlock := aBlock! !

!SortedLinkedList methodsFor: 'enumerating'!

do: aBlock 
"Evaluate aBlock with each of the receiver's elements as the argument."
	| current |
	current := firstLink.				"Initialize for traversal."
	[current isNil]					"Iterate up to and including the last node."
		whileFalse: 
			[aBlock value: current.
			current := current nextLink].	"Get the next node, if any."! !

!SortedLinkedList methodsFor: 'adding'!

add: anObject 
"Add anObject to the list at position determined by sortBlock. Return anObject."
| current newNode previous |
	"Create a node link with value anObject."
	newNode := LinkNode nodeValue: anObject.
	"If the list is empty, simply put the node at the start of the list."
	self isEmpty ifTrue: [super add: newNode. ^anObject].
	"List is not empty, traverse it from the start."
	current := self first.
	[current isNil]	"End of list?"
		whileFalse: [ 
			(sortBlock value: current value value: newNode value) "Is new node behind current node?"
				ifTrue: "Sort block succeeded, go to next list node."
					[previous := current.
					current := current nextLink]
				ifFalse: "Sort block failed, distinguish two possibilities."
					[current == firstLink
						ifTrue: "Insert new node at the start of the list."
							[newNode nextLink: firstLink.
							self addFirst: newNode]
						ifFalse: "Insert between previous and current."
							[previous nextLink: newNode.
							newNode nextLink: current].
					^anObject]].	
	"Node does not fit inside, add it at the end."
	self addLast: newNode.
	previous nextLink: newNode.
	^anObject! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SortedLinkedList class
	instanceVariableNames: ''!


!SortedLinkedList class methodsFor: 'finding'!

find: anObject 
	"Return node with value anObject or nil."!

find: anObject startingAt: aNode
	"Return node with value anObject or nil."! !

!SortedLinkedList class methodsFor: 'creation'!

new
	^super new initialize!

sortBlock: aBlock 
	"Answer a new instance of SortedCollection such that its elements 
	are sorted according to the criterion specified in aBlock."

	^super new sortBlock: aBlock! !

!SortedLinkedList class methodsFor: 'initialization'!

initialize
	"Create a default sort block."
	"SortedLinkedList initialize"

	DefaultSortBlock := [:x :y | x <= y]! !

Object subclass: #Queue
	instanceVariableNames: 'customers tellers time reporter simulationManager '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!Queue methodsFor: 'accessing'!

customers
	^customers!

customers: aCollection 
	customers := aCollection!

reporter: aReporter 
	reporter := aReporter!

simulationManager: aSimulationManager 
	simulationManager := aSimulationManager!

tellers: anInteger 
	tellers := Array new: anInteger.
	1 to: anInteger do: [:index | tellers at: index put: (Teller
				number: index
				reporter: reporter
				simulationManager: simulationManager)]!

time: anInteger 
	time := anInteger! !

!Queue methodsFor: 'adding'!

addCustomer: aCustomer 
	customers add: (aCustomer timeToQueue: time).
	reporter displayEvent: 'Time: ' , time printString , '. Queue received customer' steps: true.
	self checkTellers.! !

!Queue methodsFor: 'update'!

checkTellers
	| emptyTellers |
	(emptyTellers := tellers select: [:teller | teller customer isNil]) isEmpty
		ifTrue: [^nil]
		ifFalse: [emptyTellers first customer: customers removeFirst]!

updateTime
	time := time + 1.
	tellers do: [:teller | teller updateTime].
	[customers isEmpty not and: [self checkTellers isNil not]] whileTrue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Queue class
	instanceVariableNames: ''!


!Queue class methodsFor: 'creation'!

numberOfTellers: anInteger 
	"Queue numberOfTellers: 10"

	^self new tellers: anInteger!

numberOfTellers: anInteger reporter: aReporter simulationManager: aSimulationManager 
	"Queue numberOfTellers: 10"

	^(self new) reporter: aReporter; simulationManager: aSimulationManager; tellers: anInteger; customers: (OrderedCollection new: 10); time: 0! !

Object subclass: #BinaryTree
	instanceVariableNames: 'root isLeftBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!
BinaryTree comment:
'Class BinaryTree implements a binary tree. Nodes are added as leaves on the basis of a comparison block applied to the value of the new object and values of existing nodes. 

Instance variables 

root 		<Node>	- root node of the tree, a Node object.
isLeftBlock 	<BlockClosure>	- a two-argument block used to determine whether a new node should 
		be added to the left of an existing node. The first argument of the block is the node being 
		added, the second argument is an existing node to which the new node is being 
		compared. When the block returns true, the new node is attached as the left subnode; on 
		false, the new node is attached on the right.
'!


!BinaryTree methodsFor: 'initialization'!

root: anObject block: aTwoArgumentBlock 
	root := anObject.
	isLeftBlock := aTwoArgumentBlock! !

!BinaryTree methodsFor: 'adding '!

add: anObject 
	"Attach node with anObject as a new leaf of the tree."

	| newNode |
	newNode := Node newWithObject: anObject.
	root isNil
		ifTrue: [root := newNode]
		ifFalse: [self add: newNode toNode: root]!

add: newNode toNode: existingNode 
"Attach newNode as leaf of the appropriate subtree of existingNode."

	existingNode isNil ifTrue: [root := newNode. ^newNode].
	(isLeftBlock value: newNode value value: existingNode value)
		ifTrue: [	"Add newNode to left sub-tree."
			existingNode leftChild isNil
				ifTrue: "Attach aNode as leftChild."
					[existingNode addLeftChild: newNode]
				ifFalse: "Apply this method recursively."
					[self add: newNode toNode: existingNode leftChild]]
		ifFalse: 	"Add newNode to right sub-tree."
			[existingNode rightChild isNil
				ifTrue: " Attach aNode as rightChild."
					[existingNode addRightChild: newNode]
				ifFalse: "Apply this method recursively."
					[self add: newNode toNode: existingNode rightChild]].
		^newNode! !

!BinaryTree methodsFor: 'enumeration'!

do: aBlock 
"Evaluate aBlock for each node of the tree rooted at the root."
	self do: aBlock startingFrom: root!

do: aBlock startingFrom: aNode 
	"Evaluate aBlock for each node of the tree rooted at aNode."

	aNode isNil
		ifFalse: 
			[aBlock value: aNode.
			self do: aBlock startingFrom: aNode leftChild.
			self do: aBlock startingFrom: aNode rightChild]! !

!BinaryTree methodsFor: 'removing'!

remove: anObject ifAbsent: aBlock 
	"Delete anObject from the tree and return anObject; evaluate aBlock if not present."

	| parent theNode |
	theNode := self find: anObject.
	theNode isNil ifTrue: [^aBlock value].
	parent := theNode parent.
	parent isNil
		ifTrue: 
			["theNode must be the root."
			root leftChild isNil
				ifTrue: [root := root rightChild]
				ifFalse: 
					[self add: root rightChild toNode: root leftChild.
					root := root leftChild].
			root parent: nil]
		ifFalse: 
			[parent leftChild = theNode
				ifTrue: [self removeLeftChild]
				ifFalse: [self removeRightChild].
			self add: theNode leftChild toNode: parent.
			self add: theNode rightChild toNode: parent].
	^anObject! !

!BinaryTree methodsFor: 'finding'!

find: anObject 
	"Return node with value anObject or nil."

	^self find: anObject startingAt: root!

find: anObject startingAt: aNode 
	"Return node with value anObject or nil."

	aNode value isNil ifTrue: [^nil].
	aNode value = anObject ifTrue: [^aNode].
	^(isLeftBlock value: anObject value: aNode)
		ifTrue: [self find: anObject startingAt: aNode leftChild]
		ifFalse: [self find: anObject startingAt: aNode rightChild]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BinaryTree class
	instanceVariableNames: ''!


!BinaryTree class methodsFor: 'creation'!

withRoot: anObject withBlock: comparisonBlock 
	^self new root: (Node newWithObject: anObject)
		block: comparisonBlock! !

Link subclass: #LinkNode
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!LinkNode methodsFor: 'accessing'!

value
	^value!

value: anObject 
	value := anObject! !

!LinkNode methodsFor: 'printing'!

printOn: aStream 
	"Append to the argument, aStream, the elements of the Array enclosed by parentheses."

	aStream nextPutAll: self class name; nextPutAll: ' '; nextPutAll: self value printString! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LinkNode class
	instanceVariableNames: ''!


!LinkNode class methodsFor: 'creation'!

nodeValue: anObject 
	^self new value: anObject! !

Object subclass: #Item
	instanceVariableNames: 'name price '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!Item methodsFor: 'testing'!

isExpensive
	^price > (Currency dollars: 10 cents: 0)! !

!Item methodsFor: 'accessing'!

name
	^name!

name: aString 
	name := aString!

price
	^price!

price: anInteger 
	price := anInteger! !

!Item methodsFor: 'printing'!

displayString
	| tabString |
	tabString := String with: Character tab.
	^name , tabString , 'price: ' , price dollars printString , ' dollars   ' , (price cents rem: 100) printString , ' cents'! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Item class
	instanceVariableNames: ''!


!Item class methodsFor: 'creation'!

name: aString price: aFixedPoint
"Create new instance using aString for name and a aFixedPoint for currency."
	| dollars cents |
	dollars := aFixedPoint asInteger.			"Get the integer part of the price."
	cents := ((aFixedPoint - dollars) * 100) asInteger.	"Get the fractional part."
	^self new 
		name: aString; 
		price: (Currency dollars: dollars cents: cents)! !

Object subclass: #RandomInteger
	instanceVariableNames: 'randomGenerator lowerLimit upperLimit '
	classVariableNames: 'ClassVarName1 ClassVarName2 '
	poolDictionaries: ''
	category: 'Chapter 11'!
RandomInteger comment:
'Random number generator for integer random numbers between lower and upper limits.

randomGenerator	<Random>, the base generator
lowerLimit			<Integer>
upperLimit			<Integer>'!


!RandomInteger methodsFor: 'accessing'!

lowerLimit: anInteger 
	lowerLimit := anInteger!

next
	^(randomGenerator next * (upperLimit - lowerLimit) + lowerLimit) rounded!

randomGenerator 
	randomGenerator := Random new!

randomGenerator: aRandom 
	randomGenerator := aRandom!

upperLimit: anInteger 
	upperLimit := anInteger! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RandomInteger class
	instanceVariableNames: ''!


!RandomInteger class methodsFor: 'testing'!

test
	"RandomInteger test"

	| rg mean next min max |
	rg := RandomInteger lowerLimit: 1 upperLimit: 10.
	mean := 0.
	min := 10.
	max := 1.
	Transcript clear.
	1000
		timesRepeat: 
			[next := rg next.
			mean := mean + next.
			min := min min: next.
			max := max max: next].
	Transcript clear; show: (mean quo: 1000) printString; cr; show: min printString; cr; show: max printString! !

!RandomInteger class methodsFor: 'creation'!

lowerLimit: integer1 upperLimit: integer2 
	"RandomInteger lowerLimit: 1 upperLimit: 10"

	^(self new) lowerLimit: integer1; upperLimit: integer2; randomGenerator! !

ApplicationModel subclass: #BankSimulation
	instanceVariableNames: 'minArrival maxArrival minProcessing tellers maxProcessing length showSteps '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!BankSimulation methodsFor: 'display'!

clear
	Transcript clear!

cr
	Transcript cr!

displayEvent: aString steps: aBoolean 
	(showSteps value or: [aBoolean not])
		ifTrue: [Transcript nextPut: Character cr; nextPutAll: aString]!

displayTotalCustomers: integer1 time: integer2 
	Transcript cr; cr; show: 'Total number of customers processed in ' , integer2 printString , ' units of time is ' , integer2 printString!

flush
	Transcript flush! !

!BankSimulation methodsFor: 'actions'!

close
	self closeRequest!

help

	"This stub method was generated by UIDefiner"

	^self!

run
	((minArrival value >= 0 and: [minArrival value <= maxArrival value])
		and: [minProcessing value <= maxProcessing value and: [length value > 1 and: [tellers value > 0]]])
		ifTrue: [(SimulationManager
				tellers: tellers value
				reporter: self
				simulationLength: length value
				minProcessing: minProcessing value
				maxProcessing: maxProcessing value
				minArrival: minArrival value
				maxArrival: maxArrival value) run]! !

!BankSimulation methodsFor: 'aspects'!

length
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^length isNil
		ifTrue:
			[length := 0 asValue]
		ifFalse:
			[length]!

maxArrival
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^maxArrival isNil
		ifTrue:
			[maxArrival := 0 asValue]
		ifFalse:
			[maxArrival]!

maxProcessing
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^maxProcessing isNil
		ifTrue:
			[maxProcessing := 0 asValue]
		ifFalse:
			[maxProcessing]!

minArrival
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^minArrival isNil
		ifTrue:
			[minArrival := 0 asValue]
		ifFalse:
			[minArrival]!

minProcessing
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^minProcessing isNil
		ifTrue:
			[minProcessing := 0 asValue]
		ifFalse:
			[minProcessing]!

showSteps
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^showSteps isNil
		ifTrue:
			[showSteps := false asValue]
		ifFalse:
			[showSteps]!

tellers
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^tellers isNil
		ifTrue:
			[tellers := 0 asValue]
		ifFalse:
			[tellers]! !

!BankSimulation methodsFor: 'initialization'!

initialize
	tellers := 5 asValue.
	length := 10 asValue.
	minArrival := 1 asValue.
	maxArrival := 10 asValue.
	minProcessing := 1 asValue.
	maxProcessing := 10 asValue! !

!BankSimulation methodsFor: 'validation'!

validateLength: aController 
	self halt.
	(aController editValue isInteger and: [aController editValue > 0])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be integer and greater than 0'.
			^false]!

validateMaxArrival: aController 
	(aController editValue isInteger and: [aController editValue >= minArrival value])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be integer and greater than minimum inter-arrival time'.
			^false]!

validateMaxProcessing: aController 
	(aController editValue isInteger and: [aController editValue >= minProcessing value])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be integer and greater than minimum inter-arrival time'.
			^false]!

validateMinArrival: aController 
	(aController editValue isInteger and: [aController editValue <= maxArrival value])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be integer and smaller than maximum inter-arrival time'.
			^false]!

validateMinProcessing: aController 
	(aController editValue isInteger and: [aController editValue <= maxProcessing value])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be integer and greater than maximum processing time'.
			^false]!

validatePositiveInteger: aController 
	(aController editValue isInteger and: [aController editValue > 0])
		ifTrue: [^true]
		ifFalse: 
			[Dialog warn: 'Number must be a positive integer'.
			^false]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BankSimulation class
	instanceVariableNames: ''!


!BankSimulation class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Bank Simulation' 
			#min: #(#Point 286 271 ) 
			#max: #(#Point 286 271 ) 
			#bounds: #(#Rectangle 257 168 543 439 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#Point 8 16 ) 
					#label: 'Number of tellers' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 48 ) 
					#label: 'Length of simulation' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 80 ) 
					#label: 'Minimum inter-arrival time' ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 8 208 56 232 ) 
					#model: #run 
					#label: 'Run' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Point 8 112 ) 
					#label: 'Maximum inter-arrival time' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 16 272 40 ) 
					#model: #tellers 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validatePositiveInteger: ) 
					#alignment: #center 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 48 272 72 ) 
					#model: #length 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validatePositiveInteger: ) 
					#alignment: #center 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 80 272 104 ) 
					#model: #minArrival 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validateMinArrival: ) 
					#alignment: #center 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 112 272 136 ) 
					#model: #maxArrival 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validateMaxArrival: ) 
					#alignment: #center 
					#type: #number ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 72 208 120 232 ) 
					#model: #close 
					#label: 'Close' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Point 8 144 ) 
					#label: 'Minimum processing time' ) 
				#(#LabelSpec 
					#layout: #(#Point 8 176 ) 
					#label: 'Maximum processing time' ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 144 272 168 ) 
					#model: #minProcessing 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validateMinProcessing: ) 
					#alignment: #center 
					#type: #number ) 
				#(#InputFieldSpec 
					#layout: #(#Rectangle 176 176 272 200 ) 
					#model: #maxProcessing 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#requestFocusOutSelector: 
						#validateMaxProcessing: ) 
					#alignment: #center 
					#type: #number ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 40 240 88 264 ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#CheckBoxSpec 
					#layout: #(#Point 176 232 ) 
					#model: #showSteps 
					#label: 'Show steps' ) ) ) )! !

!BankSimulation class methodsFor: 'opening'!

opening
	"BankSimulation open"! !

Object subclass: #CustomerProducer
	instanceVariableNames: 'customer releaseTime gapGenerator processingGenerator time reporter '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!CustomerProducer methodsFor: 'initialization'!

initialize
	releaseTime := 0.
	time := 0! !

!CustomerProducer methodsFor: 'updating'!

updateTime
	time := time + 1.
	^self produceCustomer! !

!CustomerProducer methodsFor: 'accessing'!

gapGenerator: aGenerator 
	gapGenerator := aGenerator!

processingGenerator: aGenerator 
	processingGenerator := aGenerator!

reporter: aReporter 
	reporter := aReporter! !

!CustomerProducer methodsFor: 'producing customers'!

addCustomer
	customer := self newCustomer.
	releaseTime := time + gapGenerator next!

newCustomer
	^Customer newWithProcessingTime: processingGenerator next!

produceCustomer
	| oldCustomer |
	time  = releaseTime ifFalse: [^nil].
	oldCustomer := customer.
	self addCustomer.
	^oldCustomer! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CustomerProducer class
	instanceVariableNames: ''!


!CustomerProducer class methodsFor: 'creation'!

newWithGapGenerator: aRandomIntegerGenerator1 processingGenerator: aRandomIntegerGenerator2 reporter: aReporter 
	^(super new) gapGenerator: aRandomIntegerGenerator1; processingGenerator: aRandomIntegerGenerator2; reporter: aReporter; initialize! !

Object subclass: #Graph
	instanceVariableNames: 'structure '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!
Graph comment:
'I implement directed graphs consisting of vertices and weighted edges.

Instance variables

structure		<Dictionary>	association key is node, value is a set of two-element arrays consisting 
of vertex ID and the corresponding edge weight.
'!


!Graph methodsFor: 'utilities'!

connected: object1ID to: object2ID
"Is the object whose ID is object1ID connected to the object whose ID is object2ID? Return true or false."
	| active done remaining startObject found passes |
	active := OrderedCollection new: structure size. 
	active add: (startObject := structure associationAt: object1ID).
	done := Set new: structure size.
	remaining := structure - (Dictionary with: startObject).
	found := false.
	passes := 0.
	[active isEmpty or: [ found := active contains: [:vertex| (vertex value contains: [:array | (array at: 1)  = object2ID])]]] whileFalse:
		[ |currentVertex neighbors |
				passes := passes + 1.
				done add: (currentVertex := active removeFirst). 
				neighbors := self unprocessedNeighborsOf: currentVertex key from: remaining.
				active addAll: neighbors.
				remaining := remaining - (Dictionary withAll: neighbors)].
	^found ifTrue: [passes + 1] ifFalse: [nil]! !

!Graph methodsFor: 'printing'!

printOn: aStream 
	"Append to the argument, aStream, the description of a graph."

	aStream nextPutAll: self class name , ' ', structure printString! !

!Graph methodsFor: 'accessing'!

edgesFor: anObject
"Return the set of vertex-weight objects of vertex anObject." 
	^structure at: anObject!

structure: aDictionary 
	structure := aDictionary! !

!Graph methodsFor: 'private'!

unprocessedNeighborsOf: aCity from: remainingVertices 
	"Return collection of associations of all neighbors of aVertex that remain unprocessed."

	"Gather names of unprocessed neighboring cities."
	| cities unprocessed remainingCities neighboringCityArrays |
	remainingCities := remainingVertices keys.
	neighboringCityArrays := (structure at: aCity)
				select: [:setElement | remainingCities contains: [:cityEl | cityEl = (setElement at: 1)]].
	cities := neighboringCityArrays collect: [:array | array at: 1].	"Now use cities to pick associations."
	unprocessed := OrderedCollection new.
	remainingVertices keysAndValuesDo: [:key :value | (cities contains: [:city | city = key])
			ifTrue: [unprocessed add: (remainingVertices associationAt: key)]].
	^unprocessed! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Graph class
	instanceVariableNames: ''!


!Graph class methodsFor: 'creation'!

newWith: aDictionary
"Create a new graph whose structure is defined by aDictionary."
	^self new structure: aDictionary! !

Object subclass: #SimulationManager
	instanceVariableNames: 'time simulationLength queue reporter customerProducer minArrival maxArrival minProcessing maxProcessing gapGenerator totalCustomers totalTimeInQueue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!SimulationManager methodsFor: 'displaying'!

display: queueOrTeller number: anInteger event: aString
	Transcript show: queueOrTeller asString , ' ' , (queueOrTeller == #teller ifTrue: [anInteger printString]), ' ', aString! !

!SimulationManager methodsFor: 'accessing'!

maxArrival: anInteger 
	maxArrival := anInteger!

maxProcessing: anInteger 
	maxProcessing := anInteger!

minArrival: anInteger 
	minArrival := anInteger!

minProcessing: anInteger 
	minProcessing := anInteger!

queueWithTellers: anInteger 
	queue := Queue
				numberOfTellers: anInteger
				reporter: reporter
				simulationManager: self!

reporter: aBankSimulator 
	reporter := aBankSimulator!

simulationLength: anInteger 
	simulationLength := anInteger!

totalTimeInQueue 
	^totalTimeInQueue!

totalTimeInQueue: anInteger 
	totalTimeInQueue := anInteger! !

!SimulationManager methodsFor: 'initialization'!

initialize
	gapGenerator := RandomInteger lowerLimit: minArrival upperLimit: maxArrival! !

!SimulationManager methodsFor: 'printing'!

printOn: aStream 
	aStream nextPutAll: 'a SimulationManager with processing times: ' , minProcessing printString , ' - ' , maxProcessing printString , '; inter-arrival times: ' , minArrival printString , ' - ' , maxArrival printString , '; simulation length: ' , simulationLength printString! !

!SimulationManager methodsFor: 'simulation'!

run
	reporter clear.
	time := 0.
	totalTimeInQueue := 0.
	customerProducer := CustomerProducer
				newWithGapGenerator: (RandomInteger lowerLimit: minArrival upperLimit: maxArrival)
				processingGenerator: (RandomInteger lowerLimit: minProcessing upperLimit: maxProcessing)
				reporter: reporter.
	queue addCustomer: customerProducer newCustomer.
	customerProducer addCustomer.
	totalCustomers := 2.
	[time > simulationLength]
		whileFalse: 
			[| customer |
			time := time + 1.
			queue updateTime.
			customer := customerProducer updateTime.
			customer isNil
				ifFalse: 
					[queue addCustomer: customer.
					totalCustomers := totalCustomers + 1]].
	reporter cr; displayEvent: 'Total time: ' , simulationLength printString steps: false.
	reporter displayEvent: 'Total number of customers: ' , totalCustomers printString steps: false.
	reporter displayEvent: 'Average time in queue: ' , (totalTimeInQueue / totalCustomers) asFloat printString steps: false.
	reporter flush! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SimulationManager class
	instanceVariableNames: ''!


!SimulationManager class methodsFor: 'creation'!

tellers: anInteger reporter: aBankSimulation 
	"SimulationManager tellers: 10 reporter: BankSimulation new"

	^(self new) queueWithTellers: anInteger; reporter: aBankSimulation!

tellers: anInteger reporter: aBankSimulation simulationLength: simulationLength
	"SimulationManager tellers: 10 reporter: BankSimulation new"

	^(self new) queueWithTellers: anInteger; reporter: aBankSimulation!

tellers: integer1 reporter: aBankSimulation simulationLength: integer2 minProcessing: integer3 maxProcessing: integer4 minArrival: integer5 maxArrival: integer6 
	"SimulationManager tellers: 10 reporter: BankSimulation new simulationLength: 10 minProcessing: 1 
	maxProcessing: 3 minArrival: 1 maxArrival: 2"

	^(self new) reporter: aBankSimulation; queueWithTellers: integer1; simulationLength: integer2; minProcessing: integer3; maxProcessing: integer4; minArrival: integer5; maxArrival: integer6; initialize! !

Object subclass: #TextFilter
	instanceVariableNames: ''
	classVariableNames: 'InputStream MatchDictionary OutputStream '
	poolDictionaries: ''
	category: 'Chapter 11'!
TextFilter comment:
'I implement general filtering of text. To create an instance, I need the original string and two arrays consisting of strings to be matched, and replacements. Replacement values may be string or text objects or blocks. My filtering method returns the result without affection the original.

Class Variables:
InputStream	<ReadStream>		streams over input string
OutputStream	<WriteStream>		used to build filtered string
MatchDictionary	<String, Text, Block>	used to do replacements
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TextFilter class
	instanceVariableNames: ''!


!TextFilter class methodsFor: 'filtering'!

filter: aString match: matchArray replace: replacementArray 
"I filter inputString using matchArray and replacementArray, and return the resulting String ."
	"Initialization."
	MatchDictionary := Dictionary new.
	matchArray with: replacementArray do: 
		[:match :replace | MatchDictionary at: match put: (Array with: replace with: 0)].
	InputStream := ReadStream on: aString.
	OutputStream := WriteStream on: (String new: aString size).
	"Filtering."
	[InputStream atEnd] whileFalse: [self matchAndReplace].
	^OutputStream contents!

matchAndReplace
"Get next character from input stream and try to match it."
	| ch |
	ch := InputStream next.
	"Copy the input character into the output stream for now."
	OutputStream nextPut: ch.
	"Now try to match against all entries in the dictionary."
	MatchDictionary
		keysAndValuesDo: 
			[:key :value | 
			| index |
			"Get index of next character in this dictionary entry."
			index := (value at: 2) + 1.
			"Check if it equals the input character."
			ch == (key at: index)
				ifTrue: [index = key size	"We have a match. Did we match the whole 
							value?"
						ifTrue: 	"We matched the whole value."
							"Go back in output stream for replacement."
							[OutputStream skip: key size negated.
							"Put replacement in output stream."
							OutputStream nextPutAll: (value at: 1).
							"Reset match positions in all entries."
							MatchDictionary
								do: 
									[:valueArray |
									valueArray at: 2 put: 0].
									"Done with this character."
									^self]
						ifFalse: "Characters remain in match - update index."
							[value at: 2 put: index]]
				ifFalse:	"No match, reset index in this entry."
					[value at: 2 put: 0]]! !

Object subclass: #Node
	instanceVariableNames: 'value leftChild rightChild parent '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!
Node comment:
'Class Node implements nodes for class BinaryTree. Each node consists of

value		<Object>	- value of node, must understand the tree''s comparison block
leftChild  	<Node>		- root of left sub-tree of this node, an instance of Node or nil
rightChild  	<Node>		- root of right sub-tree of this node, an instance of Node or nil
parent		<Node>		- parent of this node
'!


!Node methodsFor: 'accessing'!

leftChild
	^leftChild!

parent
	^parent!

parent: aNode
	parent := aNode!

rightChild
	^rightChild!

value
	^value!

value: anObject
	value := anObject! !

!Node methodsFor: 'removing'!

removeLeftChild
	leftChild := nil!

removeRightChild
	rightChild := nil! !

!Node methodsFor: 'adding'!

addLeftChild: aNode 
	leftChild := aNode.
	aNode parent: self!

addRightChild: aNode
	rightChild := aNode.
	aNode parent: self! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Node class
	instanceVariableNames: ''!


!Node class methodsFor: 'creation'!

newWithObject: anObject 
	^self new  value: anObject! !

Object subclass: #Customer
	instanceVariableNames: 'processingTime timeToQueue '
	classVariableNames: 'MaxProcessing MinProcessing '
	poolDictionaries: ''
	category: 'Chapter 11'!


!Customer methodsFor: 'accessing'!

processingTime
	^processingTime!

processingTime: anInteger 
	processingTime := anInteger!

timeToQueue
	^timeToQueue!

timeToQueue: anInteger 
	timeToQueue := anInteger! !

!Customer methodsFor: 'printing'!

printOn: aStream 
	aStream nextPutAll: 'a Customer with processing time: ' , processingTime printString! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Customer class
	instanceVariableNames: ''!


!Customer class methodsFor: 'testing'!

test
	"Customer test"

	Transcript cr; show: (Customer newWithProcessingTime: 10)  printString.
	Transcript cr; show: (Customer newWithProcessingTime: nil)  printString! !

!Customer class methodsFor: 'creation'!

newWithProcessingTime: anInteger 
	^self new processingTime: anInteger! !

Object subclass: #Teller
	instanceVariableNames: 'customer customerReleaseTime reporter number time simulationManager '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 11'!


!Teller methodsFor: 'accessing'!

addCustomer: aCustomer atTime: anInteger 
	customer := aCustomer.
	customerReleaseTime := customer processingTime + anInteger!

customer
	^customer!

customer: aCustomer 
	customer := aCustomer.
	customerReleaseTime := customer processingTime + time.
	reporter displayEvent: 'Time: ' , time printString , '. Teller ' , number printString , ' new customer with processing time ' , customer processingTime printString steps: true!

number
	^number!

number: anInteger 
	number := anInteger!

reporter: aReporter 
	reporter := aReporter!

simulationManager: aSimulationManager 
	simulationManager := aSimulationManager!

time: anInteger 
	time := anInteger! !

!Teller methodsFor: 'printing'!

printOn: aStream 
	aStream nextPutAll: 'a Teller with '.
	customer isNil
		ifTrue: [aStream nextPutAll: 'no customer']
		ifFalse: [aStream nextPutAll: customer printString; nextPut: Character space; nextPutAll: customerReleaseTime printString]! !

!Teller methodsFor: 'update'!

updateTime
	time := time + 1.
	customer isNil ifTrue: [^self].
	customerReleaseTime = time
		ifTrue: 
			[simulationManager totalTimeInQueue: simulationManager totalTimeInQueue + time - customer timeToQueue.
			customer := nil.
			reporter isNil ifFalse: [reporter displayEvent: 'Time: ' , time printString , ' Teller ' , number printString , ' released customer' steps: true]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Teller class
	instanceVariableNames: ''!


!Teller class methodsFor: 'testing'!

test
	"Teller test"

	Transcript clear; show: ((Teller number: 3 reporter: BankSimulation new simulationManager: nil)
			customer: (Customer newWithProcessingTime: 10)) printString! !

!Teller class methodsFor: 'creation'!

number: anInteger reporter: reporter 
	"Teller number: 3 reporter: BankSimulation new"

	^(self new) number: anInteger; reporter: reporter!

number: anInteger reporter: reporter simulationManager: aSimulationManager 
	"Teller number: 3 reporter: BankSimulation new"

	^(self new) number: anInteger; reporter: reporter; time: 0; simulationManager: aSimulationManager! !
SortedLinkedList initialize!


