Object subclass: #Eraser
	instanceVariableNames: 'pad '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
Eraser comment:
'Part of the pen world. Can erase the whole or a rectangular  area on the drawing subview specified by the user and fill it with a given color. '!


!Eraser methodsFor: 'initialization'!

initialize 
	pad := ScheduledControllers activeController view model drawingView! !

!Eraser methodsFor: 'updating'!

update: anAspect
self halt.! !

!Eraser methodsFor: 'erasing'!

erase
	self eraseWithColor: ColorValue white!

eraseAll
	self eraseAllWithColor: ColorValue white!

eraseAllWithColor: aColor 
	| gc rectangle |
	rectangle := pad container bounds translatedBy: pad container bounds origin negated.
	gc := pad graphicsContext.
	gc paint: aColor.
	gc displayRectangle: rectangle.
	self savePad!

eraseWithColor: aColor
	| gc rectangle |
	rectangle := Rectangle fromUser.
	rectangle := rectangle translatedBy: (pad container bounds origin + pad container container container container globalOrigin) negated.
	gc := pad graphicsContext.
	gc paint: aColor.
	gc displayRectangle: rectangle.
	self savePad!

savePad
	| rectangle |
"insetBy: accounts for border around drawing view"
	rectangle := (pad container bounds insetBy: 1@1) translatedBy: pad container container container container globalOrigin.
	pad model image: (Screen default completeContentsOfArea: rectangle)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Eraser class
	instanceVariableNames: ''!


!Eraser class methodsFor: 'creation'!

new
	^super new initialize! !

Object subclass: #GeneralPen
	instanceVariableNames: 'angle color delay increment homePosition pad position speed up width '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
GeneralPen comment:
'A pen that works on a pad (a DrawingView) specified at the time of creation. Used as superclass in the Pen hierarchy in the pen world. Implements most of the functions of the hierarchy.

Instance variables:
	angle <Integer> - orientation: one of 0, 90, 180, 270 with 0 pointing up
	color <ColorValue> - drawing color
	delay <Integer> - milliseconds of delay between drawing consecutive points
	homePosition
	increment <Point> - describes how position is incremented during move for current angle
	pad <DrawingView> - the view on which the pen draws
	position <Point> - pen''s position on the pad (relative to pad origin)
	speed <Symbol> - #slow, #medium, #fast - speed of drawing, detemrines delay
	up  <Boolean> - is pen up (not drawing) or down (drawing)?
	width  <Integer> - width of drawing tip'!


!GeneralPen methodsFor: 'initialization'!

initializeOnPad: aPad 
	pad := aPad.
	color := ColorValue black.
	width := 2.
	homePosition := pad container bounds center - pad container bounds origin.
	position := homePosition.
	speed := #medium.
	up := false.
	angle := 90.
	self incrementFromAngle.
	self delay! !

!GeneralPen methodsFor: 'accessing'!

angle
	^angle!

color
	^color!

color: aColor 
	color := aColor!

delay
	speed == #slow ifTrue: [delay := 250].
	speed == #medium ifTrue: [delay := 40].
	speed == #fast ifTrue: [delay := 1]!

down
	up := false!

fast
	speed := #fast.
	self delay!

incrementFromAngle
	increment := angle = 0
		ifTrue: [1 @ 0]
		ifFalse: [angle = 90
				ifTrue: [0 @ -1]
				ifFalse: [angle = 180
						ifTrue: [-1 @ 0]
						ifFalse: [0 @ 1]]]!

isUp
	^up!

position
	^position!

position: aPoint
	position := aPoint!

slow
	speed := #slow.
	self delay!

speed: aSymbol 
	speed := aSymbol.
	self delay!

up
	up := true!

width: aNumber
	width := aNumber! !

!GeneralPen methodsFor: 'moving'!

home
	position := homePosition!

homePosition: aPoint
	homePosition := aPoint!

move: distance 
	distance <= 0 ifTrue: [^Dialog warn: 'The pen can only move forward'].
	(self outOfBounds: position + (distance * increment))
		ifTrue: [^Dialog warn: 'This would take the pen out of the pad'].
	up
		ifFalse: 
			[| line gc |
			gc := pad graphicsContext.
			gc lineWidth: width.
			gc paint: color.
			line := Array with: position with: position + increment.
			1 to: distance
				do: 
					[:segment | 
					gc displayLineFrom: (line at: 1)
						to: (line at: 2).
					line at: 1 put: (line at: 2).
					line at: 2 put: (line at: 2)
							+ increment.
					(Delay forMilliseconds: delay) wait]].
	position := position + (distance * increment)!

turnLeft
	angle := angle + 90 \\ 360.
	self incrementFromAngle!

turnRight
	angle := angle - 90 \\ 360.
	self incrementFromAngle! !

!GeneralPen methodsFor: 'printing'!

penProperties
	^'angle ' , angle printString , '\color ' withCRs , (ColorValue constantNameFor: color) asString , '\position ' withCRs , position printString , '\speed ' withCRs , speed asString , '\in ' withCRs , (up
				ifTrue: ['lifted position']
				ifFalse: ['painting position']) , '\width ' withCRs , width printString!

printOn: aStream 
	aStream nextPutAll: 'a ' , self class printString , ', angle ' , angle printString , ', color ' , color printString , ', position ' , position printString , ', speed ' , speed asString , ', in ' , (up
				ifTrue: ['lifted position']
				ifFalse: ['painting position']) , ', width ' , width printString! !

!GeneralPen methodsFor: 'testing'!

outOfBounds: aPoint 
	^((pad container bounds translatedBy: pad container bounds origin negated)
		containsPoint: aPoint) not! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GeneralPen class
	instanceVariableNames: ''!


!GeneralPen class methodsFor: 'creation'!

newOnPad: aPad 
	^self basicNew initializeOnPad: aPad! !

ApplicationModel subclass: #DrawingWorld
	instanceVariableNames: 'code drawingView image type '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
DrawingWorld comment:
'I provide one of several rehearsal worlds with a sketch pad and drawing pens. Different varieties provide different kinds of browsers, debuggers, inspectors, etc.

code <ValueHolder on String> - holds contents of text view
drawingView - <DrawingView> - the subview used for drawing
image <CachedImage> - holds cached image from DrawingView
type <Integer> - type (number) of this pen world'!


!DrawingWorld methodsFor: 'aspects'!

code
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^code isNil
		ifTrue:
			[code := String new asValue]
		ifFalse:
			[code]! !

!DrawingWorld methodsFor: 'accessing'!

drawingView
	^drawingView!

image: anImage 
	image := anImage!

type
	^type!

type: aNumber 
	type := aNumber! !

!DrawingWorld methodsFor: 'displaying'!

displayOn: aGraphicsContext 
	image isNil ifTrue: [^self].
	image displayOn: aGraphicsContext! !

!DrawingWorld methodsFor: 'actions'!

help
	(SimpleHelp helpString: (self class helpForType: type)) open! !

GeneralPen subclass: #Pen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
Pen comment:
'This class is the Pen that the student will inspect. It hides the unpleasant need for specifying the pad - with Pen, a new instance is produced by  message new. (Compare with supeclass GeneralPen.)'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Pen class
	instanceVariableNames: ''!


!Pen class methodsFor: 'creation'!

new
	^self newOnPad: ScheduledControllers activeController view model drawingView! !

Pen subclass: #ThickPen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
ThickPen comment:
'Pen with fixed width.'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ThickPen class
	instanceVariableNames: ''!


!ThickPen class methodsFor: 'creation'!

new
	^super new width: 3! !

Pen subclass: #ThinPen
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
ThinPen comment:
'Pen with fixed width.'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ThinPen class
	instanceVariableNames: ''!


!ThinPen class methodsFor: 'creation'!

new
	^super new width: 1! !

DrawingWorld subclass: #CommandPenWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!CommandPenWorld methodsFor: 'aspects'!

code
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^code isNil
		ifTrue:
			[code := String new asValue]
		ifFalse:
			[code]! !

!CommandPenWorld methodsFor: 'accessing'!

codeMenu
	| mb |
	mb := MenuBuilder new.
	mb add: 'find' -> #find; add: 'replace' -> #replace; line; add: 'undo' -> #undo; line; add: 'copy' -> #copySelection; add: 'cut' -> #cut; add: 'paste' -> #paste; line; add: 'do it' -> #doIt; add: 'inspect' -> #inspectIt; line; add: 'hardcopy' -> #hardcopy.
	^mb menu! !

!CommandPenWorld methodsFor: 'initialization'!

initializeWithType: typeNumber 
	self type: typeNumber.
	drawingView := DrawingView on: self.! !

!CommandPenWorld methodsFor: 'actions'!

properties
	(self builder componentAt: #codeView) widget controller inspectIt! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CommandPenWorld class
	instanceVariableNames: ''!


!CommandPenWorld class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: '' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 149 153 652 447 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 0 0.01 0 0.1 -5 0.5 0 0.9 ) 
					#component: #drawingView ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 5 0.5 0 0.1 0 0.99 0 0.9 ) 
					#name: #codeView 
					#model: #code 
					#menu: #codeMenu ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.02 ) 
					#label: 'Code workspace' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame -35 0.75 -23 0.99 35 0.75 0 0.99 ) 
					#model: #properties 
					#label: 'Properties' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#Rectangle 77 6 152 27 ) 
					#label: 'Drawing pad' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame -25 0.5 -23 0.99 25 0.5 0 0.99 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) ) ) )! !

!CommandPenWorld class methodsFor: 'opening'!

openOnType: typeNumber 
	"PenWorld openOnType: 1"

	| world |
	world := self new.
	world initializeWithType: typeNumber.
	self openOn: world! !

Object subclass: #PenAndBackground
	instanceVariableNames: 'pen background '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!PenAndBackground methodsFor: 'accessing'!

background
	^background!

background: anImage
	background := anImage!

pen
	^pen!

pen: aPen
	pen := aPen! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PenAndBackground class
	instanceVariableNames: ''!


!PenAndBackground class methodsFor: 'creation'!

withPen: aPen 
	^self new pen: aPen! !

View subclass: #DrawingView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!
DrawingView comment:
'General purpose drawing view. Leaves the actual display to its model via the displayOn: method.
'!


!DrawingView methodsFor: 'updating'!

update: anAspect with: anObject! !

!DrawingView methodsFor: 'controller accessing'!

defaultControllerClass
	^NoController! !

!DrawingView methodsFor: 'displaying'!

displayOn: aGraphicsContext 
	"sends request for display to its model"

	self model isNil ifTrue: [^self].
	self model displayOn: aGraphicsContext! !

!DrawingView methodsFor: 'model accessing'!

model: aModel 
	super model: aModel.
	self invalidate.
	self controller class == NoController ifFalse: ["Tell the controller where to send menu messages."
		self controller performer: aModel]! !

!DrawingView methodsFor: 'saving'!

savePad
	| rectangle |
"insetBy: accounts for border around drawing view"
	rectangle := (self container bounds insetBy: 1@1) translatedBy: self container container container container globalOrigin.
	self model image: (Screen default completeContentsOfArea: rectangle)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DrawingView class
	instanceVariableNames: ''!


!DrawingView class methodsFor: 'creation'!

on: aModel 
	^self new model: aModel! !

DrawingView subclass: #ClickDrawingView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!ClickDrawingView methodsFor: 'controller accessing'!

defaultControllerClass
	^ClickPenController! !

DrawingWorld subclass: #PenWorldLauncher
	instanceVariableNames: 'hasDeclarations hotAreaSide hotAreaExtent erasers pens eraser selectedItem message smalltalkStatement properties worldType '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!PenWorldLauncher methodsFor: 'initialization'!

postOpenWith: aBuilder 
	aBuilder window label: 'Pen world number ' , type printString! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PenWorldLauncher class
	instanceVariableNames: ''!


!PenWorldLauncher class methodsFor: 'help'!

helpForType: typeNumber 
	typeNumber = 1 ifTrue: [^'To send a message to an object, select the object by clicking the object button,  then send the message by clicking the message button.
As an example, to ask a pen or eraser factory to create a new pen or eraser, click the corresponding factory button and then click the new message.'].
	typeNumber = 2 ifTrue: [^'New feature - display of a preliminary form of Smalltalk statements that have the same effect as the clicked actions.

Old features:
To send a message to an object, select the object by clicking the object button,  then send the message by clicking the message button.
As an example, to ask a pen or eraser factory to create a new pen or eraser, click the corresponding factory button and then click the new message.'].
	typeNumber = 3 ifTrue: [^'New feature:
The text subview displays real Smalltalk statements that have the same effect as the clicked actions.

Old features:
To send a message to an object, select the object by clicking the object button,  then send the message by clicking the message button.
As an example, to ask a pen or eraser factory to create a new pen or eraser, click the corresponding factory button and then click the new message.
'].
	typeNumber = 4 ifTrue: [^'New feature - display of properties of objects.
To find the properties of an object (pen, pen factory, eraser, eraser factory) select the object by clicking, then click the properties button.

Old features:
To send a message to an object, select the object by clicking the object button,  then send the message by clicking the message button.
As an example, to ask a pen or eraser factory to create a new pen or eraser, click the corresponding factory button and then click the new message.'].
	typeNumber = 5 ifTrue: [^'Messages available for ThinPen and ThickPen objects:
home						as in 	pen home
homePosition: aPoint		as in	pen homePosition: 200@80
color: aColor				as in	pen color: ColorValue red
fast							as in	pen fast
medium						as in	pen medium
move: aNumber				as in 	pen move: 40
slow						as in	pen slow
turnLeft						as in 	pen turnLeft
turnRight					as in 	pen turnRight

Messages available for Erasor:
erase						as in	erasor erase
eraseAll						as in	erasor eraseAll
erase withColor: aColor		as in	erasor eraseWithColor: ColorValue green
eraseAllWithColor: aColor	as in	erasor eraseWithColor: ColorValue yellow']! !

!PenWorldLauncher class methodsFor: 'opening'!

openOnType: typeNumber 
	"PenWorld openOnType: 1"

	(typeNumber between: 1 and: 4)
		ifTrue: [ClickPenWorld openOnType: typeNumber]
		ifFalse: [CommandPenWorld openOnType: typeNumber]! !

!PenWorldLauncher class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Pen World Launcher' 
			#bounds: #(#Rectangle 300 200 756 330 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 296 20 416 46 ) 
					#model: #openPenWorld 
					#label: 'Open Pen World' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 335 58 387 84 ) 
					#model: #generalHelp 
					#label: 'Help' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 335 84 387 110 ) 
					#model: #quit 
					#label: 'Quit' 
					#defaultable: true ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 7 ) 
					#model: #worldType 
					#label: 'World 1: Simplistic' 
					#select: 1 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 31 ) 
					#model: #worldType 
					#label: 'World 2: Proper button names' 
					#select: 2 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 55 ) 
					#model: #worldType 
					#label: 'World 3: Simple Smalltalk' 
					#select: 3 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 79 ) 
					#model: #worldType 
					#label: 'World 4: Correct Smalltalk' 
					#select: 4 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 103 ) 
					#model: #worldType 
					#label: 'World 5: Code based - no buttons' 
					#select: 5 ) ) ) )! !

PenWorldLauncher subclass: #ClickPenWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!ClickPenWorld methodsFor: 'enabling/disabling'!

disable: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp disable]!

enable: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp enable]!

hide: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp beInvisible]!

hideAndDisable: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp beInvisible; enable]!

show: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp beVisible]!

showAndEnable: aCollection 
	aCollection
		do: 
			[:aSymbol | 
			| comp |
			comp := self builder componentAt: aSymbol.
			comp beVisible; enable]! !

!ClickPenWorld methodsFor: 'displaying statement'!

displayStatement
	| statementWidget index |
	statementWidget := (self builder componentAt: #smalltalkStatements) widget.
	index := statementWidget model value size.
	statementWidget controller selectAndScrollFrom: index to: index! !

!ClickPenWorld methodsFor: 'actions'!

changeColor
	| color |
	color := Dialog
				choose: 'select color'
				fromList: ColorValue constantNames
				values: ColorValue constantNames
				lines: 20
				cancel: [selectedItem pen color].
	selectedItem pen color: (ColorValue perform: color).
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'color: ColorValue ' , (ColorValue constantNameFor: selectedItem pen color)].
	self removeSelectedArrow.
	self doneDrawing!

disablePropertyButton
	type >= 4 ifTrue: [self disable: #(#propertiesButton)]!

doneDrawing
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penLabel #availableMessages #clickToSelect); hideAndDisable: #(#movePen #turnPenLeft #turnPenRight #penDown #penUp); showAndEnable: #(#eraserFactory #penFactory); show: #(#aPen #anEraser #availableObjects #selectReceiver #nilLabel).
	self showAndEnable: #(#eraserFactory).
	erasers isEmpty ifTrue: [self disable: #(#anEraser)].
	pens isEmpty ifTrue: [self disable: #(#aPen)].
	type < 4 ifTrue: [^self].
	self hide: #(#propertiesLabel).
	self disablePropertyButton!

doneErasing
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #availableMessages #eraserLabel); hideAndDisable: #(#eraseAll); showAndEnable: #(#penFactory #eraserFactory); show: #(#availableObjects #selectReceiver #nilLabel); showAndEnable: #(#anEraser).
	pens isEmpty
		ifTrue: [self show: #(#aPen)]
		ifFalse: [self showAndEnable: #(#aPen)].
	self disablePropertyButton!

drawArrows
	| graphicsContext pixMap |
	graphicsContext := drawingView graphicsContext.
	pens
		do: 
			[:penAndBackground | 
			pixMap := Pixmap extent: hotAreaExtent.
			pixMap graphicsContext
				copyArea: (0 @ 0 extent: hotAreaExtent)
				from: graphicsContext
				sourceOffset: penAndBackground pen position - hotAreaSide
				destinationOffset: 0 @ 0.
			penAndBackground background: pixMap].
	pens do: [:penAndBackground | graphicsContext displayPolygon: (self arrowPointsFor: penAndBackground) at: penAndBackground pen position
"(Circle center: penAndBackground pen position radius: hotAreaSide)
			displayStrokedOn: graphicsContext"].
	drawingView savePad!

enablePropertyButton
	type >= 4 ifTrue: [self enable: #(#propertiesButton)]!

eraseAll
	| string |
	selectedItem eraseAll.
	self doneErasing.
	type = 1 ifTrue: [^self].
	string := smalltalkStatement value isEmpty
				ifTrue: ['']
				ifFalse: ['.\' withCRs].
	string := string , 'eraser eraseAll'.
	self updateStatementWith: string!

eraserFactory
	self hideAndDisable: #(#penFactory #eraserFactory #aPen #anEraser); hide: #(#selectReceiver #availableObjects #nilLabel); show: #(#selectMessage #availableMessages #eraserFactoryLabel #availableMessages); showAndEnable: #(#newEraser).
	selectedItem := Eraser.
	self enablePropertyButton!

eraserFactoryProperties
	self showPropertiesView.
	properties value: 'Eraser factory (class Eraser) does not have any properties.'!

eraserProperties
	self showPropertiesView.
	properties value: 'an Eraser (instance of class Eraser) does not have any properties.'!

eraserSelection
	selectedItem := eraser.
	self hideAndDisable: #(#penFactory #aPen #anEraser #eraserFactory); hide: #(#selectReceiver #availableObjects #nilLabel); show: #(#selectMessage #availableMessages #selectedObject #eraserLabel #availableMessages); showAndEnable: #(#eraseAll).
	self enablePropertyButton!

findItem
	| comp |
	self drawArrows.
	comp := self builder componentAt: #clickToSelect.
	comp invalidateRectangle: comp bounds repairNow: true.
	5 timesRepeat: 
			[self show: #(#clickToSelect).
			comp invalidateRectangle: comp bounds repairNow: true.
			(Delay forMilliseconds: 100) wait.
			self hide: #(#clickToSelect).
			comp invalidateRectangle: comp bounds repairNow: true.
			(Delay forMilliseconds: 50) wait].
	self show: #(#clickToSelect).
	comp invalidateRectangle: comp bounds repairNow: true.
	self getPen.
	self removeArrows.
	self hide: #(#clickToSelect)!

hideMessages
	self hide: #(#eraserLabel #selectMessage #availableMessages #penLabel #availableMessages #clickToSelect); hideAndDisable: #(#eraseAll #movePen #turnPenLeft #turnPenRight #penDown #penUp).
	self disablePropertyButton!

hidePropertiesView
	self hide: #(#propertiesLabel); hideAndDisable: #(#properties #hidePropertiesButton).
	self showAndEnable: #(#eraserFactory #penFactory #smalltalkStatements #propertiesButton); show: #(#selectedObject #availableObjects #selectReceiver #nilLabel #smalltalkStatement #propertiesButtonLabel).
	self show: #(#anEraser).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self show: #(#aPen).
	pens isEmpty
		ifTrue: [self disable: #(#aPen)]
		ifFalse: [self enable: #(#aPen)].
	self hideMessages!

movePen
	| steps |
	steps := (Dialog request: 'Enter number of steps (>0)' initialAnswer: '') asNumber.
	steps <= 0 ifTrue: [steps := nil].
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'move: ' , steps printString].
	self removeSelectedArrow.
	steps isNil ifFalse: [selectedItem pen move: steps].
	self doneDrawing.
	drawingView savePad.!

newBlackPen
	| string |
	selectedItem := PenAndBackground withPen: ThinPen new.
	selectedItem pen color: (ColorValue black).
	pens add: selectedItem.
	type > 1
		ifTrue: 
			[string := smalltalkStatement value isEmpty
						ifTrue: ['']
						ifFalse: ['.\' withCRs].
			string := string , (type = 2
							ifTrue: ['Pen']
							ifFalse: [type > 2
									ifTrue: 
										[| number |
										number := (pens indexOf: selectedItem) printString.
										'pen' , number , ' := Pen']]).
			self updateStatementWith: string , ' newBlackPen'].
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penFactoryLabel #availableMessages); hideAndDisable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen); showAndEnable: #(#eraserFactory #penFactory #aPen); show: #(#anEraser #availableObjects #selectReceiver #nilLabel).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self disablePropertyButton!

newBluePen
	| string |
	selectedItem := PenAndBackground withPen: ThinPen new.
	selectedItem pen color: (ColorValue blue).
	pens add: selectedItem.
	type > 1
		ifTrue: 
			[string := smalltalkStatement value isEmpty
						ifTrue: ['']
						ifFalse: ['.\' withCRs].
			string := string , (type = 2
							ifTrue: ['Pen']
							ifFalse: [type > 2
									ifTrue: 
										[| number |
										number := (pens indexOf: selectedItem) printString.
										'pen' , number , ' := Pen']]).
			self updateStatementWith: string , ' newBluePen'].
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penFactoryLabel #availableMessages); hideAndDisable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen); showAndEnable: #(#eraserFactory #penFactory #aPen); show: #(#anEraser #availableObjects #selectReceiver #nilLabel).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self disablePropertyButton!

newDeclaration
"Change text in text view if declaration required"
	| string |
	(pens isEmpty and: [erasers isEmpty])
		ifTrue: [^''].
	string := WriteStream on: (String new: 10).
	string nextPutAll: '| '.
	1 to: erasers size do: [:index | string nextPutAll: 'eraser' , index printString , ' '].
	1 to: pens size do: [:index | string nextPutAll: 'pen' , index printString , ' '].
	string nextPutAll: '|'; cr.
	hasDeclarations
		ifTrue: 
			[| position |
			position := smalltalkStatement value findString: '\' withCRs startingAt: 1.
			string nextPutAll: (smalltalkStatement value copyFrom: position + 1 to: smalltalkStatement value size)].
	hasDeclarations := true.
	^string contents!

newEraser
	| string |
	eraser := selectedItem := Eraser new.
	erasers add: selectedItem.
	self hide: #(#selectMessage #availableMessages #eraserFactoryLabel #availableMessages); hideAndDisable: #(#newEraser); show: #(#availableObjects #selectReceiver #anEraser #nilLabel #penFactory #eraserFactory).
	pens isEmpty
		ifTrue: [self show: #(#aPen)]
		ifFalse: [self showAndEnable: #(#aPen)].
	type = 1 ifTrue: [^self].
	string := smalltalkStatement value isEmpty
				ifTrue: ['']
				ifFalse: ['.\' withCRs].
	string := string , (type = 2
					ifTrue: ['Eraser new']
					ifFalse: 
						[| number |
						number := (erasers indexOf: selectedItem) printString.
						type > 2 ifTrue: ['eraser ' , number , ' :=  Eraser new']]).
	self updateStatementWith: string.
	self disablePropertyButton!

newGreenPen
	| string |
	selectedItem := PenAndBackground withPen: ThinPen new.
	selectedItem pen color: (ColorValue green).
	pens add: selectedItem.
	type > 1
		ifTrue: 
			[string := smalltalkStatement value isEmpty
						ifTrue: ['']
						ifFalse: ['.\' withCRs].
			string := string , (type = 2
							ifTrue: ['Pen']
							ifFalse: [type > 2
									ifTrue: 
										[| number |
										number := (pens indexOf: selectedItem) printString.
										'pen' , number , ' := Pen']]).
			self updateStatementWith: string , ' newGreenPen'].
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penFactoryLabel #availableMessages); hideAndDisable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen); showAndEnable: #(#eraserFactory #penFactory #aPen); show: #(#anEraser #availableObjects #selectReceiver #nilLabel).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self disablePropertyButton!

newPen
	| string |
	selectedItem := PenAndBackground withPen: ThinPen new.
	pens add: selectedItem.
	type > 1
		ifTrue: 
			[string := smalltalkStatement value isEmpty
						ifTrue: ['']
						ifFalse: ['.\' withCRs].
			string := string , (type = 2
							ifTrue: ['Pen']
							ifFalse: [type > 2
									ifTrue: 
										[| number |
										number := (pens indexOf: selectedItem) printString.
										'pen' , number , ' := Pen']]).
			self updateStatementWith: string , ' new'].
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penFactoryLabel #availableMessages); hideAndDisable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen); showAndEnable: #(#eraserFactory #penFactory #aPen); show: #(#anEraser #availableObjects #selectReceiver #nilLabel).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self disablePropertyButton!

newRedPen
	| string |
	selectedItem := PenAndBackground withPen: ThinPen new.
	selectedItem pen color: (ColorValue red).
	pens add: selectedItem.
	type > 1
		ifTrue: 
			[string := smalltalkStatement value isEmpty
						ifTrue: ['']
						ifFalse: ['.\' withCRs].
			string := string , (type = 2
							ifTrue: ['Pen']
							ifFalse: [type > 2
									ifTrue: 
										[| number |
										number := (pens indexOf: selectedItem) printString.
										'pen' , number , ' := Pen']]).
			self updateStatementWith: string , ' newRedPen'].
	selectedItem := nil.
	self hide: #(#selectMessage #availableMessages #penFactoryLabel #availableMessages); hideAndDisable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen); showAndEnable: #(#eraserFactory #penFactory #aPen); show: #(#anEraser #availableObjects #selectReceiver #nilLabel).
	erasers isEmpty
		ifTrue: [self disable: #(#anEraser)]
		ifFalse: [self enable: #(#anEraser)].
	self disablePropertyButton!

penDown
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'penDown'].
	selectedItem pen down.
	self removeSelectedArrow.
	self doneDrawing!

penFactory
	self hideAndDisable: #(#penFactory #aPen #anEraser); hide: #(#penFactory #eraserFactory #availableObjects #selectReceiver #nilLabel); show: #(#selectMessage #availableMessages #selectedObject #penFactoryLabel #availableMessages); showAndEnable: #(#newBlackPen #newBluePen #newGreenPen #newRedPen).
	selectedItem := PenAndBackground.
	self enablePropertyButton!

penFactoryProperties
	self showPropertiesView.
	properties value: 'Pen factory (class Pen) does not have any properties.'!

penProperties
	self showPropertiesView.
	properties value: selectedItem pen penProperties.!

penSelection
	self hide: #(#selectReceiver).
	pens size = 1
		ifTrue: 
			[self drawArrows.
			selectedItem := pens first]
		ifFalse: [self findItem].
	self hideAndDisable: #(#penFactory #aPen #anEraser); hide: #(#availableObjects #eraserFactory #nilLabel); show: #(#selectMessage #availableMessages #selectedObject #penLabel #availableMessages); showAndEnable: #(#movePen #turnPenLeft #turnPenRight #penDown #penUp).
	self enablePropertyButton!

penUp
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'penUp'].
	selectedItem pen up.
	self removeSelectedArrow.
	self doneDrawing.!

removeArrows
	| graphicsContext |
	graphicsContext := drawingView graphicsContext.
	pens do: [:penAndBackground | penAndBackground == selectedItem ifFalse: [graphicsContext
				copyArea: (0 @ 0 extent: penAndBackground background extent)
				from: penAndBackground background graphicsContext
				sourceOffset: 0 @ 0
				destinationOffset: penAndBackground pen position - (hotAreaSide @ hotAreaSide)]].
	drawingView savePad.
	drawingView invalidateRectangle: drawingView bounds repairNow: true!

removeSelectedArrow
	| graphicsContext |
"	pens size = 1 ifTrue: [^self]."
	graphicsContext := drawingView graphicsContext.
	graphicsContext
		copyArea: (0 @ 0 extent: selectedItem background extent)
		from: selectedItem background graphicsContext
		sourceOffset: 0 @ 0
		destinationOffset: selectedItem pen position - (hotAreaSide @ hotAreaSide).
	drawingView savePad.
	drawingView invalidateRectangle: drawingView bounds repairNow: true!

showPropertiesView
	self hide: #(#selectedObject #availableObjects #selectReceiver #selectMessage #nilLabel #smalltalkStatement #penFactoryLabel #eraserFactoryLabel #availableMessages); hideAndDisable: #(#penFactory  #newBlackPen #newBluePen #newGreenPen #newRedPen #eraserFactory #anEraser #smalltalkStatements #eraseAll #newEraser #aPen #penUp #penDown #turnPenLeft #turnPenRight #movePen); showAndEnable: #(#properties #hidePropertiesButton); show: #(#propertiesLabel)!

turnPenLeft
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'turnLeft'].
	selectedItem pen turnLeft.
	self removeSelectedArrow.
	self doneDrawing!

turnPenRight
	type > 1 ifTrue: [self updateStatementWith: self currentPenString , 'turnRight'].
	selectedItem pen turnRight.
	self removeSelectedArrow.
	self doneDrawing!

updateStatementWith: aString 
	| newText |
	type < 3
		ifTrue: [newText := smalltalkStatement value]
		ifFalse: [newText := self newDeclaration].
	smalltalkStatement value: newText , aString.
	self displayStatement! !

!ClickPenWorld methodsFor: 'initialization'!

initializeWithType: typeNumber 
	self type: typeNumber.
	drawingView := ClickDrawingView on: self.
	pens := OrderedCollection new.
	erasers := OrderedCollection new.
	message := '' asValue.
	hotAreaSide := 6.
	hotAreaExtent := 2 * hotAreaSide @ (2 * hotAreaSide).
	smalltalkStatement := '' asValue.
	properties := '' asValue.
	hasDeclarations := false! !

!ClickPenWorld methodsFor: 'pen selection'!

getPen
	| controller pen point sensor |
	controller := drawingView controller.
	sensor := controller sensor.
	[sensor redButtonPressed and: [pen isNil not]]
		whileFalse: 
			[point := sensor cursorPoint.
			pen := pens detect: [:aPen | (Rectangle origin: aPen pen position - hotAreaSide corner: aPen pen position + hotAreaSide)
							containsPoint: point]
						ifNone: [nil].
			pen isNil
				ifTrue: [Cursor normal show]
				ifFalse: [Cursor crossHair show]].
	Cursor normal show.
	selectedItem := pen! !

!ClickPenWorld methodsFor: 'accessing'!

arrowPointsFor: aPenAndBackground 
	| angle array side |
	side := hotAreaSide * 2 // 3.
	angle := aPenAndBackground pen angle.
	angle = 0 ifTrue: [array := Array
					with: side negated @ side negated
					with: 0 @ 0
					with: side negated @ side].
	angle = 90 ifTrue: [array := Array
					with: side negated @ side
					with: 0 @ 0
					with: side @ side].
	angle = 180 ifTrue: [array := Array
					with: side @ side
					with: 0 @ 0
					with: side @ side negated].
	angle = 270 ifTrue: [array := Array
					with: side negated @ side negated
					with: 0 @ 0
					with: side @ side negated].
	^array!

currentPenString
	| string |
	type = 1 ifTrue: [^self].
	string := smalltalkStatement value isEmpty
				ifTrue: ['']
				ifFalse: ['.\' withCRs].
	string := string , 'pen' , (type = 2
					ifTrue: ['']
					ifFalse: [type > 2 ifTrue: [(pens indexOf: selectedItem) printString]]).
	^string , ' '!

message
	^message!

message: aString
	message value: aString!

pens
	^pens!

selectedItem: aPen 
	selectedItem := aPen! !

!ClickPenWorld methodsFor: 'aspects'!

properties
	^properties!

propertiesButton
	self hide: #(#selectMessage #availableMessages #clickToSelect #propertiesButtonLabel); hideAndDisable: #(#eraseAll #movePen #turnPenLeft #turnPenRight #penDown #penUp #propertiesButton).
	(selectedItem isMemberOf: PenAndBackground)
		ifTrue: [self penProperties].
	(selectedItem isMemberOf: PenAndBackground class)
		ifTrue: [self penFactoryProperties].
	(selectedItem isMemberOf: Eraser)
		ifTrue: [self eraserProperties].
	(selectedItem isMemberOf: Eraser class)
		ifTrue: [self eraserFactoryProperties].
	selectedItem := nil.!

smalltalkStatement
	^smalltalkStatement! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ClickPenWorld class
	instanceVariableNames: ''!


!ClickPenWorld class methodsFor: 'interface specs'!

windowSpec1
	"UIPainter new openOnClass: self andSelector: #windowSpec1"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: '' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 258 282 761 576 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.6 0 0.51 ) 
					#name: #selectReceiver 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Select receiver object' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.51 ) 
					#name: #clickToSelect 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) ) 
					#label: 'Select a pen' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.62 0 0.18 0 0.81 25 0.18 ) 
					#name: #newEraser 
					#flags: 56 
					#model: #newEraser 
					#label: 'new eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.6 0 0.1 ) 
					#name: #availableMessages 
					#flags: 16 
					#label: 'Available messages' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.63 0 0.18 0 0.82 25 0.18 ) 
					#name: #eraseAll 
					#flags: 56 
					#model: #eraseAll 
					#label: 'erase all' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.62 0 0.51 ) 
					#name: #selectMessage 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Select message' ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 0 0.01 0 0.1 -5 0.5 0 0.98 ) 
					#component: #drawingView ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 388 0.1 99 0.09 438 0.1 0 0.503401 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #movePen 
					#flags: 56 
					#model: #movePen 
					#label: 'move' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.18 0 0.97 25 0.18 ) 
					#name: #turnPenLeft 
					#flags: 56 
					#model: #turnPenLeft 
					#label: 'turn left' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.28 0 0.97 25 0.28 ) 
					#name: #turnPenRight 
					#flags: 56 
					#model: #turnPenRight 
					#label: 'turn right' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #penUp 
					#flags: 56 
					#model: #penUp 
					#label: 'pen up' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #penFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penFactory 
					#label: 'pen factory' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #eraserFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserFactory 
					#label: 'eraser factory' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserFactoryLabel 
					#flags: 16 
					#label: 'an Eraser factory' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.53 0 0.02 ) 
					#name: #selectedObject 
					#label: 'Selected object:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #penFactoryLabel 
					#flags: 16 
					#label: 'a Pen factory' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.62 0 0.1 ) 
					#name: #availableObjects 
					#label: 'Available objects' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.18 0 0.94 25 0.18 ) 
					#name: #aPen 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penSelection 
					#label: 'pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.28 0 0.94 25 0.28 ) 
					#name: #anEraser 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserSelection 
					#label: 'eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserLabel 
					#flags: 16 
					#label: 'eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.75 0 0.02 ) 
					#name: #penLabel 
					#flags: 16 
					#label: 'pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.73 0 0.02 ) 
					#name: #nilLabel 
					#label: 'nil' ) 
				#(#LabelSpec 
					#layout: #(#Rectangle 101 6 184 24 ) 
					#label: 'Drawing pad' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.508946 0 0.384354 0 0.687873 56 0.28 ) 
					#name: #penDown 
					#flags: 56 
					#model: #penDown 
					#label: 'pen down' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.763419 0 0.289116 0 0.962227 59 0.18 ) 
					#name: #newRedPen 
					#flags: 56 
					#model: #newRedPen 
					#label: 'new red pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.508946 0 0.289116 0 0.707753 59 0.18 ) 
					#name: #newGreenPen 
					#flags: 56 
					#model: #newGreenPen 
					#label: 'new green pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.763419 0 0.180272 0 0.962227 27 0.18 ) 
					#name: #newBluePen 
					#flags: 56 
					#model: #newBluePen 
					#label: 'new blue pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.508946 0 0.180272 0 0.707753 27 0.18 ) 
					#name: #newBlackPen 
					#flags: 56 
					#model: #newBlackPen 
					#label: 'new black pen' 
					#defaultable: true ) ) ) )!

windowSpec2
	"UIPainter new openOnClass: self andSelector: #windowSpec2"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: '' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 271 283 774 577 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #movePen 
					#flags: 56 
					#model: #movePen 
					#label: 'move:' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #penFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penFactory 
					#label: 'class Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.63 0 0.18 0 0.82 25 0.18 ) 
					#name: #eraseAll 
					#flags: 56 
					#model: #eraseAll 
					#label: 'eraseAll' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.62 0 0.51 ) 
					#name: #selectReceiver 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Select receiver object' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.51 ) 
					#name: #selectMessage 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Select message' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.51 ) 
					#name: #clickToSelect 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) ) 
					#label: 'Select a pen' ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 0 0.01 0 0.1 -5 0.5 0 0.98 ) 
					#component: #drawingView ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.2 0 0.02 ) 
					#label: 'Drawing pad' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.874751 0 0.445578 440 0.1 0 0.52381 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.502982 0 0.180272 0 0.691849 25 0.18 ) 
					#name: #newEraser 
					#flags: 56 
					#model: #newEraser 
					#label: 'new' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.18 0 0.97 25 0.18 ) 
					#name: #turnPenLeft 
					#flags: 56 
					#model: #turnPenLeft 
					#label: 'turnLeft' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.28 0 0.97 25 0.28 ) 
					#name: #turnPenRight 
					#flags: 56 
					#model: #turnPenRight 
					#label: 'turnRight' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #penUp 
					#flags: 56 
					#model: #penUp 
					#label: 'up' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #eraserFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserFactory 
					#label: 'class Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserFactoryLabel 
					#flags: 16 
					#label: 'class Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.53 0 0.02 ) 
					#name: #selectedObject 
					#label: 'Selected object:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #penFactoryLabel 
					#flags: 16 
					#label: 'class Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.61 0 0.1 ) 
					#name: #availableObjects 
					#label: 'Available objects' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.18 0 0.94 25 0.18 ) 
					#name: #aPen 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penSelection 
					#label: 'a Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.28 0 0.94 25 0.28 ) 
					#name: #anEraser 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserSelection 
					#label: 'an Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserLabel 
					#flags: 16 
					#label: 'an Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72167 0 0.0204082 ) 
					#name: #penLabel 
					#flags: 16 
					#label: 'a Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.73 0 0.02 ) 
					#name: #nilLabel 
					#label: 'nil' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.6 0 0.1 ) 
					#name: #availableMessages 
					#flags: 16 
					#label: 'Available messages' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.63 0 0.61 ) 
					#name: #smalltalkStatement 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Smalltalk statement' ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.68 0 0.98 0 0.98 ) 
					#name: #smalltalkStatements 
					#model: #smalltalkStatement 
					#style: #systemDefault 
					#isReadOnly: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.500994 0 0.180272 0 0.691849 25 0.18 ) 
					#name: #newBlackPen 
					#flags: 56 
					#model: #newBlackPen 
					#label: 'newBlackPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.500994 0 0.289116 0 0.691849 57 0.18 ) 
					#name: #newGreenPen 
					#flags: 56 
					#model: #newGreenPen 
					#label: 'newGreenPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.771372 0 0.180272 0 0.962227 25 0.18 ) 
					#name: #newBluePen 
					#flags: 56 
					#model: #newBluePen 
					#label: 'newBluePen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.771372 0 0.289116 0 0.962227 57 0.18 ) 
					#name: #newRedPen 
					#flags: 56 
					#model: #newRedPen 
					#label: 'newRedPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.510934 0 0.387755 0 0.689861 57 0.28 ) 
					#name: #penDown 
					#flags: 56 
					#model: #penDown 
					#label: 'down' 
					#defaultable: true ) ) ) )!

windowSpec3
	"UIPainter new openOnClass: self andSelector: #windowSpec3"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: '' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 278 282 781 576 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.63 0 0.18 0 0.82 25 0.18 ) 
					#name: #eraseAll 
					#flags: 56 
					#model: #eraseAll 
					#label: 'eraseAll' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.62 0 0.51 ) 
					#name: #selectReceiver 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Select receiver object' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.51 ) 
					#name: #selectMessage 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Select message' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.65 0 0.51 ) 
					#name: #clickToSelect 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) ) 
					#label: 'Select a pen' ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 0 0.01 0 0.1 -5 0.5 0 0.98 ) 
					#component: #drawingView ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.2 0 0.02 ) 
					#label: 'Drawing pad' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.63 0 0.18 0 0.82 25 0.18 ) 
					#name: #newEraser 
					#flags: 56 
					#model: #newEraser 
					#label: 'new' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #movePen 
					#flags: 56 
					#model: #movePen 
					#label: 'move:' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.18 0 0.97 25 0.18 ) 
					#name: #turnPenLeft 
					#flags: 56 
					#model: #turnPenLeft 
					#label: 'turnLeft' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.78 0 0.28 0 0.97 25 0.28 ) 
					#name: #turnPenRight 
					#flags: 56 
					#model: #turnPenRight 
					#label: 'turnRight' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #penUp 
					#flags: 56 
					#model: #penUp 
					#label: 'up' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.18 0 0.69 25 0.18 ) 
					#name: #penFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penFactory 
					#label: 'class Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.28 0 0.69 25 0.28 ) 
					#name: #eraserFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserFactory 
					#label: 'class Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserFactoryLabel 
					#flags: 16 
					#label: 'class Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.53 0 0.02 ) 
					#name: #selectedObject 
					#label: 'Selected object:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #penFactoryLabel 
					#flags: 16 
					#label: 'class Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.61 0 0.1 ) 
					#name: #availableObjects 
					#label: 'Available objects' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.18 0 0.94 25 0.18 ) 
					#name: #aPen 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penSelection 
					#label: 'a Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.75 0 0.28 0 0.94 25 0.28 ) 
					#name: #anEraser 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserSelection 
					#label: 'an Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.72 0 0.02 ) 
					#name: #eraserLabel 
					#flags: 16 
					#label: 'an Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.75 0 0.02 ) 
					#name: #penLabel 
					#flags: 16 
					#label: 'a Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.73 0 0.02 ) 
					#name: #nilLabel 
					#label: 'nil' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.6 0 0.1 ) 
					#name: #availableMessages 
					#flags: 16 
					#label: 'Available messages' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.63 0 0.61 ) 
					#name: #smalltalkStatement 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Smalltalk statement' ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.51 0 0.68 0 0.98 0 0.98 ) 
					#name: #smalltalkStatements 
					#model: #smalltalkStatement 
					#style: #systemDefault 
					#isReadOnly: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 389 0.1 102 0.09 439 0.1 0 0.513605 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.500994 0 0.180272 0 0.691849 25 0.18 ) 
					#name: #newBlackPen 
					#flags: 56 
					#model: #newBlackPen 
					#label: 'newBlackPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.755467 0 0.180272 0 0.946322 25 0.18 ) 
					#name: #newBluePen 
					#flags: 56 
					#model: #newBluePen 
					#label: 'newBluePen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.500994 0 0.289116 0 0.691849 57 0.18 ) 
					#name: #newGreenPen 
					#flags: 56 
					#model: #newGreenPen 
					#label: 'newGreenPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.755467 0 0.289116 0 0.946322 57 0.18 ) 
					#name: #newRedPen 
					#flags: 56 
					#model: #newRedPen 
					#label: 'newRedPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.510934 0 0.387755 0 0.689861 57 0.28 ) 
					#name: #penDown 
					#flags: 56 
					#model: #penDown 
					#label: 'down' 
					#defaultable: true ) ) ) )!

windowSpec4
	"UIPainter new openOnClass: self andSelector: #windowSpec4"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: '' 
			#min: #(#Point 40 20 ) 
			#bounds: #(#Rectangle 208 292 781 586 ) ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.18 0 0.82 0 0.9 ) 
					#name: #properties 
					#flags: 60 
					#model: #properties 
					#isReadOnly: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.499127 0 0.510204 ) 
					#name: #selectReceiver 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#label: 'Select receiver object' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.52 0 0.510204 ) 
					#name: #clickToSelect 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #blue ) ) 
					#label: 'Select a pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.48 0 0.1 ) 
					#name: #availableMessages 
					#flags: 16 
					#label: 'Available messages' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.5 0 0.18 0 0.66 25 0.18 ) 
					#name: #newEraser 
					#flags: 56 
					#model: #newEraser 
					#label: 'new' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.5 0 0.18 0 0.66 25 0.18 ) 
					#name: #eraseAll 
					#flags: 56 
					#model: #eraseAll 
					#label: 'eraseAll' 
					#defaultable: true ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 0 0.01 0 0.1 -5 0.39 0 0.98 ) 
					#component: #drawingView ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.2 0 0.02 ) 
					#label: 'Drawing pad' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 439 0.1 121 0.09 489 0.1 0 0.578231 ) 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #red ) ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.85 0 0.1 ) 
					#name: #propertiesButtonLabel 
					#label: 'Properties' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.83 0 0.18 0 0.99 25 0.18 ) 
					#name: #propertiesButton 
					#flags: 40 
					#model: #propertiesButton 
					#label: 'properties' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.528796 0 0.510204 ) 
					#name: #selectMessage 
					#flags: 16 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Select message' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.180272 0 0.55 25 0.18 ) 
					#name: #movePen 
					#flags: 56 
					#model: #movePen 
					#label: 'move:' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.62 0 0.18 0 0.78 25 0.18 ) 
					#name: #turnPenLeft 
					#flags: 56 
					#model: #turnPenLeft 
					#label: 'turnLeft' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.62 0 0.28 0 0.78 25 0.28 ) 
					#name: #turnPenRight 
					#flags: 56 
					#model: #turnPenRight 
					#label: 'turnRight' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.28 0 0.55 25 0.28 ) 
					#name: #penUp 
					#flags: 56 
					#model: #penUp 
					#label: 'up' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.180272 0 0.55 25 0.18 ) 
					#name: #penFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penFactory 
					#label: 'class Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.28 0 0.55 25 0.28 ) 
					#name: #eraserFactory 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserFactory 
					#label: 'class Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.58 0 0.02 ) 
					#name: #eraserFactoryLabel 
					#flags: 16 
					#label: 'class Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.41 0 0.02 ) 
					#name: #selectedObject 
					#label: 'Selected object:' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.58 0 0.02 ) 
					#name: #penFactoryLabel 
					#flags: 16 
					#label: 'class Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.48 0 0.1 ) 
					#name: #availableObjects 
					#label: 'Available objects' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.62 0 0.18 0 0.78 25 0.18 ) 
					#name: #aPen 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #penSelection 
					#label: 'a Pen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.62 0 0.28 0 0.78 25 0.28 ) 
					#name: #anEraser 
					#flags: 40 
					#colors: 
					#(#LookPreferences 
						#setBackgroundColor: #(#ColorValue 8191 8191 6143 ) ) 
					#model: #eraserSelection 
					#label: 'an Eraser' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.600349 0 0.0204082 ) 
					#name: #eraserLabel 
					#flags: 16 
					#label: 'an Eraser' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.58 0 0.02 ) 
					#name: #penLabel 
					#flags: 16 
					#label: 'a Pen' ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.609075 0 0.0204082 ) 
					#name: #nilLabel 
					#label: 'nil' ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.39 0 0.625 0 0.98 0 0.98 ) 
					#name: #smalltalkStatements 
					#model: #smalltalkStatement 
					#style: #systemDefault 
					#isReadOnly: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.49 0 0.91 0 0.73 25 0.91 ) 
					#name: #hidePropertiesButton 
					#flags: 24 
					#model: #hidePropertiesView 
					#label: 'Close properties view' 
					#defaultable: true ) 
				#(#LabelSpec 
					#layout: #(#LayoutOrigin 0 0.53 0 0.1 ) 
					#name: #propertiesLabel 
					#flags: 16 
					#label: 'Properties' ) 
				#(#LabelSpec 
					#layout: #(#Rectangle 295 166 409 187 ) 
					#name: #smalltalkStatement 
					#colors: 
					#(#LookPreferences 
						#setForegroundColor: #(#ColorValue #black ) ) 
					#label: 'Smalltalk statement' ) 
				#(#LabelSpec 
					#layout: #(#Rectangle 505 125 538 144 ) 
					#name: #helpButtonLabel 
					#label: 'Help' ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.38918 0 0.387755 0 0.549738 57 0.28 ) 
					#name: #penDown 
					#flags: 56 
					#model: #penDown 
					#label: 'down' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.38918 0 0.180272 0 0.547993 25 0.18 ) 
					#name: #newBlackPen 
					#flags: 56 
					#model: #newBlackPen 
					#label: 'newBlackPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.38918 0 0.289116 0 0.547993 57 0.18 ) 
					#name: #newGreenPen 
					#flags: 56 
					#model: #newGreenPen 
					#label: 'newGreenPen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.626527 0 0.180272 0 0.78534 25 0.18 ) 
					#name: #newBluePen 
					#flags: 56 
					#model: #newBluePen 
					#label: 'newBluePen' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#LayoutFrame 0 0.626527 0 0.289116 0 0.78534 57 0.18 ) 
					#name: #newRedPen 
					#flags: 56 
					#model: #newRedPen 
					#label: 'newRedPen' 
					#defaultable: true ) ) ) )! !

!ClickPenWorld class methodsFor: 'opening'!

openOnType: typeNumber 
	"PenWorld openOnType: 1"
	| world |
	world := self new.
	world initializeWithType: typeNumber.
	self openOn: world withSpec: ('windowSpec' , typeNumber printString) asSymbol! !

ControllerWithMenu subclass: #ClickPenController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!ClickPenController methodsFor: 'locating pen'!

getPen
	| pens pen point |
	pens := self model pens.
	[self sensor redButtonPressed and: [pen isNil not]]
		whileFalse: 
			[point := self sensor cursorPoint.
			pen := pens detect: [:aPen | (Rectangle origin: aPen pen position - 5 corner: aPen pen position + 5)
							containsPoint: point]
						ifNone: [nil].
			pen isNil
				ifTrue: [Cursor normal show]
				ifFalse: [Cursor crossHair show]].
	Cursor normal show.
	self model selectedItem: pen! !

!ClickPenController methodsFor: 'control activity'!

controlActivity
	"self sensor redButtonPressed ifTrue: [self model activePenAt: self sensor cursorPoint]"! !

ApplicationModel subclass: #PenWorld
	instanceVariableNames: 'worldType '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 3'!


!PenWorld methodsFor: 'actions'!

help

	(SimpleHelp helpString:
'Use this launcher to open one of five ''worlds'' designed to introduce the concepts of object oriented thinking and Smalltalk.
To proceed, select a radio button on the left and then click "Open Selected Pen World".

World 1: Create colored pens and click buttons to draw lines.
World 2: Same as World 1 but button names follow Smalltalk rules.
World 3: Same as World 2 but displays simplified Smalltalk code corresponding to buttons.
World 4: Same as World 3 but displays correct Smalltalk code.
World 5: No more buttons - do everything by writing Smalltalk code.') open!

openPenWorld

	PenWorldLauncher openOnType: worldType value! !

!PenWorld methodsFor: 'aspects'!

worldType
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^worldType isNil
		ifTrue:
			[worldType := nil asValue]
		ifFalse:
			[worldType]! !

!PenWorld methodsFor: 'initialization'!

initialize
	worldType := 1 asValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PenWorld class
	instanceVariableNames: ''!


!PenWorld class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Pen World Launcher' 
			#bounds: #(#Rectangle 172 235 628 365 ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 275 20 447 46 ) 
					#model: #openPenWorld 
					#label: 'Open Selected Pen World' 
					#defaultable: true ) 
				#(#ActionButtonSpec 
					#layout: #(#Rectangle 335 58 387 84 ) 
					#model: #help 
					#label: 'Help' 
					#defaultable: true ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 7 ) 
					#model: #worldType 
					#label: 'World 1: Simplistic' 
					#select: 1 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 31 ) 
					#model: #worldType 
					#label: 'World 2: Proper button names' 
					#select: 2 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 55 ) 
					#model: #worldType 
					#label: 'World 3: Simple Smalltalk' 
					#select: 3 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 79 ) 
					#model: #worldType 
					#label: 'World 4: Correct Smalltalk' 
					#select: 4 ) 
				#(#RadioButtonSpec 
					#layout: #(#Point 11 103 ) 
					#model: #worldType 
					#label: 'World 5: Code based - no buttons' 
					#select: 5 ) ) ) )! !

