Dictionary variableSubclass: #MultiValueDictionary
	instanceVariableNames: 'collectionClass '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 9'!


!MultiValueDictionary methodsFor: 'accessing'!

at: key put: anObject 
	"If key already exists, add anObject to its value collection, otherwise create new collection with 
	anObject."

	(self includesKey: key)
		ifTrue: [(self at: key)
				add: anObject]
		ifFalse: [super at: key put: ((collectionClass new) add: anObject; yourself)]!

collectionClass
	^collectionClass!

collectionClass: aClass 
	collectionClass := aClass! !

!MultiValueDictionary methodsFor: 'printing'!

printOn: aStream 
	"Print dictionary associations followed by name of value class."

	super printOn: aStream.
	aStream nextPutAll: ', '; print: collectionClass! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MultiValueDictionary class
	instanceVariableNames: ''!


!MultiValueDictionary class methodsFor: 'creation'!

new: size onClass: aClass 
	"Create new MultiValueDictionary with specified capacity on the specified class."

	^(super new: size)
		collectionClass: aClass!

newOnClass: aClass 
	"Create new MultiValueDictionary with default capacity on the specified class."

	^(self new: 2)
		collectionClass: aClass! !

Object subclass: #TwoWayDictionary
	instanceVariableNames: 'lang1 lang2 lang1lang2 lang2lang1 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 9'!
TwoWayDictionary comment:
'Class TwoWayDictionary implements two complementary dictionaries such as English-French and French-English. In each of the two dictionaries, one key may have any number of meanings (values). Entering a key-value pair into one of the two dictionaries automatically updates the other dictionary. 
Instance variables are

<lang1>		- name of the first language, a String
<lang2>		- name of the second language, a String
< lang1lang2 > 	- one language dictionary, a MultiValueDictionary on SortedCollection
< lang2lang1>	- the other language dictionary, a MultiValueDictionary on SortedCollection

In both dictionaries, keys are strings (words in one language) and values are sorted collections of strings - equivalents of the key word in the other language.
'!


!TwoWayDictionary methodsFor: 'adding'!

add: anL1L2Association 
	"Update both dictionaries."

	| lang1String lang2String |
	lang1String := anL1L2Association key.
	lang2String := anL1L2Association value.
	lang1lang2 at: lang1String put: lang2String.
	lang2lang1 at: lang2String put: lang1String.
	^anL1L2Association!

addLang1: lang1String lang2: lang2String 
	"Update both dictionaries."

	lang1lang2 at: lang1String put: lang2String.
	lang2lang1 at: lang2String put: lang1String.
	^lang1String -> lang2String! !

!TwoWayDictionary methodsFor: 'initialization'!

initialize
	lang1lang2 := MultiValueDictionary newOnClass: SortedCollection.
	lang2lang1 := MultiValueDictionary newOnClass: SortedCollection! !

!TwoWayDictionary methodsFor: 'removing'!

language1Delete: aString 
"Remove language1 word aString from both dictionaries."
	| meanings |
	"Get all language2 meanings for aString."
	meanings := lang1lang2 at: aString.
	"Remove aString from all associations in language 2 containing them."
	meanings do: [:meaning| (lang2lang1 at: meaning) remove: aString].
	"Remove association with key aString from language 1. We know that aString is an existing key."
	lang1lang2 removeKey: aString!

language2Delete: aString 
"Remove language1 word aString from both dictionaries."
	| meanings |
	"Get all language1 meanings for aString."
	meanings := lang2lang1 at: aString.
	"Remove aString from all associations in language1 containing them."
	meanings do: [:meaning| (lang1lang2 at: meaning) remove: aString].
	"Remove association with key aString from language 2. We know that aString is an existing key."
	lang2lang1 removeKey: aString! !

!TwoWayDictionary methodsFor: 'accessing'!

lang1
	^lang1!

lang1: aString 
	lang1 := aString!

lang1lang2
	^lang1lang2!

lang2
	^lang2!

lang2: aString 
	lang2 := aString!

lang2lang1
	^lang2lang1! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TwoWayDictionary class
	instanceVariableNames: ''!


!TwoWayDictionary class methodsFor: 'creation'!

on: lang1String and: lang2String 
	^(self new) initialize; lang1: lang1String; lang2: lang2String! !

Object subclass: #FSA
	instanceVariableNames: 'currentState endStates initialState stateDictionary failBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Chapter 9'!
FSA comment:
'I implement an abstract finite state automaton.

Instance variables:

stateDictionary	<Dictionary> 	- keys are states, values are blocks; each block returns a next state
endStates 	<Set>		- a Set of terminal states, subset of stateDictionary keys
current state 	<anObject>	- current state object - a key in stateDictionary
initialState 	<anObject>	- initial state - a key in stateDictionary
failBlock		<aBlockClosure>	- block executed on any attempt to make an illegal transition; must return a next state
'!


!FSA methodsFor: 'operation'!

executeState
	"Execute action and make transition."

	^(stateDictionary at: currentState) value!

run
	"Iterate until FSA enters one of the terminal states."

	currentState := initialState.
	
	[currentState := self executeState.
	(stateDictionary includesKey: currentState)
		ifFalse: [currentState := failBlock value].
	endStates includes: currentState] whileFalse.
	self executeState! !

!FSA methodsFor: 'creation'!

onDictionary: aDictionary initialState: anInitialState endStates: aSet failBlock: aBlock 
	"Initialize FSA."

	stateDictionary := aDictionary.
	initialState := anInitialState.
	currentState := anInitialState.
	endStates := aSet.
	failBlock := aBlock! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FSA class
	instanceVariableNames: ''!


!FSA class methodsFor: 'creation'!

onDictionary: aStateDictionary initialState: anInitialObject endStates: aSet failBlock: aBlock 
	"Create an initialized FSA."

	^self new
		onDictionary: aStateDictionary
		initialState: anInitialObject
		endStates: aSet
		failBlock: aBlock! !

